﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Navigation;
using System.Windows.Markup;

namespace DynamicNavigation
{
    public class DynamicPage : Page, INavigate
    {
        public DynamicPage()
        {
        }

        public new NavigationContext NavigationContext
        {
            get
            {
                return Shim.NavigationContext;
            }
        }

        public new NavigationService NavigationService
        {
            get
            {
                return Shim.NavigationService;
            }
        }

        public new NavigationCacheMode NavigationCacheMode
        {
            get
            {
                return Shim.NavigationCacheMode;
            }
            set
            {
                base.NavigationCacheMode = value;
                Shim.NavigationCacheMode = value;
            }
        }

        public new string Title
        {
            get
            {
                return Shim.Title;
            }
            set
            {
                base.Title = value;
                Shim.Title = value;
            }
        }

        private Page shim;
        internal Page Shim
        {
            get
            {
                return shim ?? this;
            }
            set
            {
                shim = value;
                if (shim != null)
                {
                    shim.Title = base.Title;
                    shim.CacheMode = base.CacheMode;
                }
            }
        }

        internal void IOnNavigatedTo(NavigationEventArgs e)
        {
            OnNavigatedTo(e);
        }

        public bool RelativeLinks
        {
            get { return (bool)GetValue(RelativeLinksProperty); }
            set { SetValue(RelativeLinksProperty, value); }
        }

        // Using a DependencyProperty as the backing store for RelativeLinks.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty RelativeLinksProperty =
            DependencyProperty.Register("RelativeLinks", typeof(bool), typeof(DynamicPage), new PropertyMetadata(true));

        private static readonly Uri basePlaceHolderUri = new Uri("none:///", UriKind.Absolute);

        #region INavigate Members

        public bool Navigate(Uri navigateUri)
        {
            string original = navigateUri.OriginalString;
            if (RelativeLinks && !navigateUri.IsAbsoluteUri && !original.StartsWith("/"))
            {
                Uri result;
                if (NavigationService.CurrentSource.IsAbsoluteUri)
                {
                    result = new Uri(NavigationService.CurrentSource, navigateUri);
                }
                else
                {
                    Uri baseUri = new Uri(basePlaceHolderUri, NavigationService.CurrentSource);
                    result = new Uri("/" + basePlaceHolderUri.MakeRelativeUri(new Uri(baseUri, navigateUri)).OriginalString,
                        UriKind.Relative);
                }
                return NavigationService.Navigate(result);
            }
            else
            {
                return NavigationService.Navigate(navigateUri);
            }
        }

        #endregion
    }
}
